#include <Adafruit_NeoPixel.h>
#define LED_PIN 2
Adafruit_NeoPixel pixels(1, LED_PIN, NEO_GRB + NEO_KHZ800);

// Please tune the following value if the clock gains or loses.
// Theoretically, standard of this value is 60000.
#define MILLIS_PER_MIN 60071 // milliseconds per a minute

#define STEPS_PER_ROTATION 4096 // steps for a full turn of minute rotor

#define POS 14
#define OFFSET 40

// minimum wait for a single step of stepper
#define DELAY 1

// ports used to control the stepper motor
// if your motor rotate to the opposite direction, 
// change the order as {4, 5, 6, 7};
//int port[4] = {7, 6, 5, 4};
//int port[4] = {10, 8, 1, 0};
int port[4] = {21, 19, 18, 9};

// sequence of stepper motor control
int seq[8][4] = {
  {  LOW, HIGH, HIGH,  LOW},
  {  LOW,  LOW, HIGH,  LOW},
  {  LOW,  LOW, HIGH, HIGH},
  {  LOW,  LOW,  LOW, HIGH},
  { HIGH,  LOW,  LOW, HIGH},
  { HIGH,  LOW,  LOW,  LOW},
  { HIGH, HIGH,  LOW,  LOW},
  {  LOW, HIGH,  LOW,  LOW}
};



void rotate(long step) {
  static long phase = 0;
  long i, j;
  long delta = (step > 0) ? 1 : 7;
  int dt = 20;
  
  step = (step > 0) ? step : -step;
  for(j = 0; j < step; j++) {
    phase = (phase + delta) % 8;
    for(i = 0; i < 4; i++) {
      digitalWrite(port[i], seq[phase][i]);
    }
    delay(dt);
    if(dt > DELAY) {
      dt--; // accelerate
    }
  }
  // power cut
  for(i = 0; i < 4; i++) {
    digitalWrite(port[i], LOW);
  }
}

void setup() {
  pinMode(port[0], OUTPUT);
  pinMode(port[1], OUTPUT);
  pinMode(port[2], OUTPUT);
  pinMode(port[3], OUTPUT);
  pixels.setPixelColor(0, pixels.Color(255, 0, 0));    //Colorメソッド内の引数の順番は赤、緑、青
  pixels.show();
  rotate(-STEPS_PER_ROTATION); // reverse rotation to endstop
  rotate(STEPS_PER_ROTATION/2 + OFFSET); // show 0
  pixels.setPixelColor(0, pixels.Color(0, 255, 0));    //Colorメソッド内の引数の順番は赤、緑、青
  pixels.show();
}

#define APPROACH_RUN 0

void advanceStep(int i) {
  static long prev_pos = 0;
  static long count, pos;

  count += i;
  pos = (count * STEPS_PER_ROTATION) / POS;
  rotate(-APPROACH_RUN);
  rotate(pos + APPROACH_RUN - prev_pos);
  prev_pos = pos;
}

//int order[POS] = {0, 10, 5, 3, 8, 6, 2, 13, 9, 1, 11, 4, 12, 7};
//int order[POS] = {0, 13, 7, 10, 8, 12, 6, 3, 11, 5, 1, 4, 2, 9};
int order[POS] = {0, 2, 7, 10, 4, 12, 5, 8, 3, 11, 1, 6, 9, 13};

void showNum(int n) {
  int i, newPos = 0, advance;
  static int current = 0;

  for(i = 0; i < POS; i++) {
    if(order[i] == n)
      newPos = i;
  }
  advance = (newPos + POS - current) % POS;
  advanceStep(advance);
  current = newPos;
}

void loop() {
  long min;
  static long prev_min;
  static int n;

  n = (n + 1) % POS;
  showNum(n);
  delay(200);
#if 0
min = millis() / MILLIS_PER_MIN;
  if(digitalRead(16) == LOW || min != prev_min) {
    advanceStep(1);
  }
  prev_min = min;
#endif
}
